package xic.analysis;

import java.util.HashSet;
import java.util.Set;

public class Dominators extends DataFlowAnalysis {
	/// Value class used during dominator analysis
	public static class DomValue implements Value {
		/* List of variable names */
		private Set<CFGNode> dominators;

		public DomValue() {
			dominators = new HashSet<CFGNode>();
		}

		public DomValue(Set<CFGNode> dominators) {
			this.dominators = new HashSet<CFGNode>(dominators);
		}

		public Value copy() {
			return new DomValue(dominators);
		}

		public void meet(Value v) {
			dominators.retainAll(((DomValue) v).dominators);
		}

		public String toString() {
			return dominators.toString();
		}

		public int hashCode() {
			return dominators.hashCode();
		}
		
		public Set<CFGNode> dominators() {
			return dominators;
		}
		
		public boolean equals(Object obj) {
			if (!(obj instanceof DomValue))
				return false;
			return dominators.equals(((DomValue) obj).dominators);
		}

		public void setNullMeet() {
			dominators.clear();
		}
	};

	/// Dataflow-related information associated with each node
	public static class DomFlowInfo extends FlowInfo {
		public CFGNode node;

		public DomFlowInfo(CFGNode node) {
			entry = new DomValue();
			exit = new DomValue();
			this.node = node;
		}
	};

	public Dominators(Set<CFGNode> nodes) {
		super(nodes);

		dir = Direction.FORWARD;
		top = new DomValue(nodes);
	}

	protected FlowInfo createFlowInfo(CFGNode node) {
		return new DomFlowInfo(node);
	}

	public Set<CFGNode> getDominators(CFGNode n) {
		return ((DomValue) ((DomFlowInfo) getFlowInfo(n)).exit).dominators;
	}

	public void F(FlowInfo flowInfo) {
		DomFlowInfo f = (DomFlowInfo) flowInfo;
		Set<CFGNode> entry = ((DomValue) f.entry).dominators;
		Set<CFGNode> exit = ((DomValue) f.exit).dominators;
		
		exit.clear();
		exit.addAll(entry);
		exit.add(f.node);
	}
}
