package xic.analysis;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import xic.analysis.PropagatedExpressions.PEValue;
import xic.ir.IRExpr;
import xic.ir.IRStmt;


public class Substitutable extends LiveVariables {
	private PropagatedExpressions prop;
	private LiveVariables live;
	private int MAX_LIVE_VARS;

	public Substitutable(Set<CFGNode> nodes, CFGNode entry, 
			PropagatedExpressions prop,
			LiveVariables live, int max_live_vars) {
		super(nodes);
		this.MAX_LIVE_VARS = max_live_vars;
		this.prop = prop;
		this.live = live;
		initFlowInfo();
		/* empty set */
		this.top = new LVValue();
		dir = Direction.BACKWARD;
	}

	public class SubstitutableFlowInfo extends LVFlowInfo {
		boolean tooManyLV = false;
		Map<String,IRExpr> uniqueExprs;
		Set<String> propOutTemps;
		CFGNode node;

		public SubstitutableFlowInfo(Object o) {
			super(((CFGNode)o).value());
			node = (CFGNode)o;
			entry = new LVValue();
			exit = new LVValue();
			PEValue propOut = prop.out(node);
			propOutTemps = propOut.temps();
			uniqueExprs = propOut.uniquePairs();
			tooManyLV = live.getLiveVariablesBefore(node).size() > MAX_LIVE_VARS;
		}

		protected FlowInfo createFlowInfo(CFGNode node) {
			return new SubstitutableFlowInfo(node);
		}
	};
	

	public Set<String> in(CFGNode node) {
		SubstitutableFlowInfo info = (SubstitutableFlowInfo) flowInfo.get(node);
		return ((LVValue) info.exit).vars();
	}
	
	public Set<String> out(CFGNode node) {
		SubstitutableFlowInfo info = (SubstitutableFlowInfo) flowInfo.get(node);
		return ((LVValue) info.entry).vars();
	}

	public void F(FlowInfo f) {
		SubstitutableFlowInfo fi = (SubstitutableFlowInfo) f;
		Set<String> entry = ((LVValue) f.entry).vars();
		Set<String> exit = ((LVValue) f.exit).vars();

		exit.clear();
		exit.addAll(entry);
		exit.removeAll(fi.def);
		exit.retainAll(fi.propOutTemps);
		
		if(fi.tooManyLV ||  true) { ///XXX remove true 
			if(fi.node.value() instanceof IRStmt) {
				Set<String> candidates = new HashSet<String>();
				IRStmt e = (IRStmt)fi.node.value();
				e.findReadTemps(candidates);
				for(String temp : candidates) {
					System.out.println("Candidate = " + temp +", unique " + fi.uniqueExprs);
					if (fi.uniqueExprs.containsKey(temp)) {
						IRExpr uniqueExpr = fi.uniqueExprs.get(temp);
						
						/* Remove propagated temporaries from the live variables after the current node */
						Set<String> actual_liveOut = new HashSet<String>(live.getLiveVariablesAfter(fi.node));
						actual_liveOut.removeAll(fi.propOutTemps);
						System.out.println("**** " + fi.propOutTemps);
					
						Set<String> used = new HashSet<String>();
						
						uniqueExpr.findReadTemps(used);
						
						/* Live variables after A contain temporaries referenced by the expression */
						if (actual_liveOut.containsAll(used)) {
							exit.add(temp);
						}
					}
				}
			}
		}
	}

	protected FlowInfo createFlowInfo(CFGNode node) {
		return new SubstitutableFlowInfo(node);
	}
}
