package xic.ast;

import java.util.ArrayList;

import xic.MutablePosition;
import xic.PositionImpl;
import xic.Trackable;
import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.AbstractSyntaxNode;
import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Base class of all AST nodes. Stores information common to
 * all of them, such the position record.
 */
public abstract class ASTNode implements AbstractSyntaxNode, Trackable, Copyable {
	protected MutablePosition position;

	public ASTNode(Position position) {
		if (position != null)
			this.position = new PositionImpl(position);
	}

	/**
	 * Return a position record encompassing the tokens of this AST node
	 */
	public MutablePosition position() {
		return position;
	}
	
	/**
	 * Extends the right-hand side of the position record to
	 * match that of the given parameter.
	 * This is internally used when adding children to an AST node,
	 * e.g. when iteratively building a <code>Block</code>. When
	 * the internal position record is set to null, it is overwritten
	 * by the parameter.
	 */
	protected void extendPositionTo(Position right) {
		if (position == null || right == null) {
			if (right == null)
				return;
			position = new PositionImpl(right);
		}
		position.setLineEnd(right.lineEnd());
		position.setColumnEnd(right.columnEnd());
	}

	/**
	 *  Return an empty list by default
	 */
	public Iterable<VisualizableTreeNode> children() {
		return new ArrayList<VisualizableTreeNode>();
	}

	/**
	 * Traversal logic - to be implemented by all AST nodes
	 */
	public abstract ASTNode visitChildren(NodeVisitor v);
}
