package xic.ast;

import java.util.Arrays;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Represents a binary expression of two arbitrary 
 * sub-expressions. The particular operation type 
 * is denoted by an element of the <code>Type</code> 
 * enumeration.
 */
public class BinaryExpr extends Expr {
	public enum OpType {
		PLUS,
		MINUS,
		TIMES,
		DIVIDE,
		MODULO,
		EQUAL,
		NOT_EQUAL,
		LEQ,
		GEQ,
		GT,
		LT,
		OR,
		AND
	};
	private Expr left, right;
	private OpType opType;

	public BinaryExpr(Position pos, OpType type, Expr left, Expr right) {
		super(pos);
		this.left = left;
		this.right = right;
		this.opType = type;
	}

	public Expr left() {
		return left;
	}
	
	public Expr right() {
		return right;
	}

	public OpType opType() {
		return opType;
	}
	
	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			left, right
		});
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	Expr left = (Expr) v.visit(this, this.left);
		Expr right = (Expr) v.visit(this, this.right);

		if (left != this.left || right != this.right)
			return new BinaryExpr(position, opType, left, right);

		return this;
	}
    
    public Copyable copy() {
    	Expr expr = new BinaryExpr(position, opType, left, right);
    	expr.type = type;
    	expr.dataSource = dataSource;
    	return expr;
    }

	public String label() {
		switch (opType) {
			case PLUS: return "op[+]";
			case MINUS: return "op[-]";
			case TIMES: return "op[*]";
			case DIVIDE: return "op[/]";
			case MODULO: return "op[%]";
			case EQUAL: return "op[==]";
			case NOT_EQUAL: return "op[!=]";
			case LEQ: return "op[<=]";
			case GEQ: return "op[>=]";
			case LT: return "op[<]";
			case GT: return "op[>]";
			case OR: return "op[|]";
			case AND: return "op[&]";
			default: return "invalid";
		}
	}
}
