package xic.ast;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

public class ClassDecl extends Decl implements Typed {
	private String name, parentName;
	private TypeNode type;
	private Map<String, TypeNode> symbolTable;
	private DeclBlock decls;
	private boolean imported;

	public ClassDecl(Position pos, String className, String parentName,
			DeclBlock db, boolean imported) {
		super(pos);
		this.name = className;
		this.parentName = parentName;
		this.decls = db;
		this.imported = imported;
		type = new ClassType(pos, className, parentName, imported);
	}

	public ClassDecl(Position position, String className,
			String parentName, DeclBlock decls,
			Map<String, TypeNode> symbolTable, boolean imported) {
		super(position);
		this.name = className;
		this.parentName = parentName;
		this.decls = decls;
		this.symbolTable = symbolTable;
		this.imported = imported;
	}


	public String name() {
		return name;
	}
	
	public String parentName() {
		return parentName;
	}

	public boolean isDerived() {
		return parentName != null;
	}
	
	public boolean isImported() {
		return imported;
	}

	public TypeNode getType() {
		return type;
	}
	
	public DeclBlock declBlock() {
		return decls;
	}

	public Typed setType(TypeNode type) {
		if (type == this.type)
			return this;

		ClassDecl e = (ClassDecl) copy();
		e.type = type;
		return e;
	}

	public Map<String, TypeNode> getSymbolTable() {
		return symbolTable;
	}

	public ClassDecl setSymbolTable(Map<String, TypeNode> symbolTable) {
		if (symbolTable == this.symbolTable)
			return this;
		ClassDecl f = (ClassDecl) copy();
		f.symbolTable = symbolTable;
		return f;
	}


    public ASTNode visitChildren(NodeVisitor v) {
    	DeclBlock decls = (DeclBlock) v.visit(this, this.decls);

		if (decls != this.decls)
			return new ClassDecl(position, name, parentName, decls, imported);

		return this;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			decls
		});
	}

	public List<String> getNames() {
		return Arrays.asList(new String[] { name });
	}

	public String label() {
		if (isDerived())
			return "classDecl " + name + " extends " + parentName;
		else
			return "classDecl " + name;
	}

	public Copyable copy() {
		ClassDecl cd = new ClassDecl(position, name, parentName, decls, symbolTable, imported);
		cd.type = type;
		return cd;
	}
}
