package xic.ast;

import java.util.Arrays;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Converts an expression into a statement - this
 * is only valid if the expression is of 'unit' type, 
 * e.g. a call to a procedure.
 */
public class ExprStmt extends Stmt {
	private Expr expr;
	
	public ExprStmt(Position pos, Expr expr) {
		super(pos);
		this.expr = expr;
	}

	public Expr expr() {
		return expr;
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	Expr expr = (Expr) v.visit(this, this.expr);

		if (expr != this.expr)
			return new ExprStmt(position, expr);

		return this;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			expr
		});
	}

	public String label() {
		return "exprStmt";
	}

	public Copyable copy() {
		return new ExprStmt(position, this.expr);
	}
}
