package xic.ast;

import xic.ErrorSink;
import xic.visitors.NodeVisitor;
import edu.cornell.cs.cs4120.xi.CompilationException;
import edu.cornell.cs.cs4120.xi.Position;


/**
 * Represents the type of a function
 */
public class FuncType extends TypeNode {
	private TypeNode paramType;
	private TypeNode retType;

	public FuncType(Position pos, TypeNode paramType, TypeNode retType) {
		super(pos);
		this.paramType = paramType;
		this.retType = retType;
	}

	public Copyable copy() {
		return new FuncType(position(), paramType, retType);
	}
	
	public TypeNode paramType() {
		return paramType;
	}
	
	public TypeNode retType() {
		return retType;
	}

	public boolean isSubtypeOf(TypeNode node) {
		// Unused for now
		return false;
	}

	public ASTNode visitChildren(NodeVisitor v) {
		// Will never occur as part of an AST
		throw new UnsupportedOperationException();
	}

	public boolean isIdenticalType(TypeNode node) {
		if (!(node instanceof FuncType))
			return false;

		FuncType other = (FuncType) node;
		return retType.isIdenticalType(other.retType)
			&& paramType.isIdenticalType(other.paramType);
	}

	public FuncType setRetType(TypeNode retType) {
		if (retType == this.retType)
			return this;
		FuncType f = (FuncType) copy();
		f.retType = retType;
		return f;
	}

	public FuncType setParamType(TypeNode paramType) {
		if (paramType == this.paramType)
			return this;
		FuncType f = (FuncType) copy();
		f.paramType = paramType;
		return f;
	}

	public String label() {
		return "funcType";
	}

	public String toString() {
		return paramType.toString() + " -> " + retType.toString();
	}

	public int size() {
		throw new UnsupportedOperationException("FuncType.size()");
	}
	
	public void match(TypeNode node, ErrorSink sink) {
		if (node instanceof FuncType) {
			FuncType ft = (FuncType) node;

			this.paramType.match(ft.paramType, sink);
			this.retType.match(ft.retType,sink);
		}
		else
			sink.addError( new CompilationException("Type unmatch, "+
					"expected: "+this.label()+", got: "+node.label(), node.position()));
	}
}
