package xic.ast;

import xic.ErrorSink;
import xic.visitors.NodeVisitor;
import edu.cornell.cs.cs4120.xi.CompilationException;
import edu.cornell.cs.cs4120.xi.Position;

public class NullType extends TypeNode {
	/* Static types without position information */
	public static final NullType NULL_TYPE =
		new NullType(null);
	
	/**
	 * "Null" type
	 */
	public NullType(Position pos) {
			super(pos);
	}

	public String label() {
			return "nullType";
	}
	
	public boolean isSubtypeOf(TypeNode node) {
		if(node instanceof NullType
				|| node instanceof ArrayType
				|| node instanceof ClassType)
			return true;
		 else if (node instanceof TupleType) {
				TupleType tnode = (TupleType) node;
				if (tnode.elementTypes().size() == 1)
					return isSubtypeOf(tnode.elementTypes().get(0));
				else
					return false;
		 } else 
			 return false;
	}
	
	public ASTNode visitChildren(NodeVisitor v) {
		return this;
	}
	

	public boolean isIdenticalType(TypeNode node) {
		return node instanceof NullType;
	}

	public Copyable copy() {
		return new NullType (position);
	}
	
	public String toString() {
		return "null";
	}
	
	public int size() {
		return 1;
	}
	
	public void match(TypeNode node, ErrorSink sink) {
		if (!(node instanceof NullType))
			sink.addError(new CompilationException("Unmatched type, excepted: "
					+this.label()+", got: "+node.label(), node.position()));
	}
}