package xic.ir;

import java.util.Arrays;
import java.util.Set;

import xic.ast.ASTNode;
import xic.ast.Copyable;
import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

public class IRCJump extends IRStmt {
	private IRExpr expr;
	private String trueLabel, falseLabel;

	public IRCJump(IRExpr expr, String trueLabel, String falseLabel) {
		this.expr = expr;
		this.trueLabel = trueLabel;
		this.falseLabel = falseLabel;
	}

	public IRExpr expr() {
		return expr;
	}
	
	public String trueLabel() {
		return trueLabel;
	}

	public String falseLabel() {
		return falseLabel;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			expr
		});
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	IRExpr expr = (IRExpr) v.visit(this, this.expr);

		if (expr != this.expr)
			return new IRCJump(expr, trueLabel, falseLabel);

		return this;
	}

	public String label() {
		return "CJUMP";
	}
	
	public boolean containsCalls() {
		return expr.containsCalls();
	}

	public boolean containsMemWrites() {
		return expr.containsMemWrites();
	}

	public boolean containsMemReads() {
		return expr.containsMemReads();
	}

	public void findReadTemps(Set<String> tempNames) {
		expr.findReadTemps(tempNames);
	}

	public void findWrittenTemps(Set<String> tempNames) {
		expr.findWrittenTemps(tempNames);
	}

	public int nodeCount() {
		return 1 + expr.nodeCount();
	}

	public int computeMaximumCallArguments() {
		return 0;
	}

	public int computeMaximumCallResults() {
		return 0;		
	}

	public Copyable copy() {
		return new IRCJump(expr, trueLabel, falseLabel);
	}
	
	public Copyable deepCopy() {
		return new IRCJump((IRExpr) expr.deepCopy(), trueLabel, falseLabel);
	}
	
	public boolean hasTrueLabel() {
		return trueLabel != null;
	}

	public boolean hasFalseLabel() {
		return falseLabel != null;
	}

	public boolean equalsTree(Object object) {
		if (!(object instanceof IRCJump))
			return false;
		IRCJump other = (IRCJump) object;
		if (!other.expr.equalsTree(expr))
			return false;
		if (hasTrueLabel() && !other.hasTrueLabel()
			|| !hasTrueLabel() && other.hasTrueLabel())
			return false;
		if (hasFalseLabel() && !other.hasFalseLabel()
			|| !hasFalseLabel() && other.hasFalseLabel())
			return false;
		if (hasTrueLabel() && !other.trueLabel.equals(trueLabel))
			return false;
		if (hasFalseLabel() && !other.falseLabel.equals(falseLabel))
			return false;
		return true;
	}
	
	public int treeHashCode() {
		return 17 + expr.treeHashCode() + trueLabel.hashCode() * 37 
			+ falseLabel.hashCode() * 37 * 37;
	}
}
