package xic.ir;

import java.util.Arrays;
import java.util.Set;

import xic.ast.ASTNode;
import xic.ast.Copyable;
import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

public class IRESeq extends IRExpr {
	private IRStmt stmt;
	private IRExpr expr;
	
	public IRESeq(IRStmt stmt, IRExpr expr) {
		this.stmt = stmt;
		this.expr = expr;
	}

	public String label() {
		return "ESEQ";
	}

	public IRStmt stmt() {
		return stmt;
	}

	public IRExpr expr() {
		return expr;
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	IRStmt stmt = (IRStmt) v.visit(this, this.stmt);
    	IRExpr expr = (IRExpr) v.visit(this, this.expr);

		if (expr != this.expr || stmt != this.stmt)
			return new IRESeq(stmt, expr);

		return this;
	}

    public boolean containsCalls() {
    	return stmt.containsCalls() || expr.containsCalls();
    }

    public boolean containsMemReads() {
    	return stmt.containsMemReads() || expr.containsMemReads();
    }

    public boolean containsMemWrites() {
    	return stmt.containsMemWrites() || expr.containsMemWrites();
    }

    public void findReadTemps(Set<String> tempNames) {
    	stmt.findReadTemps(tempNames);
    	expr.findReadTemps(tempNames);
    }
    
    public void findWrittenTemps(Set<String> tempNames) {
    	stmt.findWrittenTemps(tempNames);
    	expr.findWrittenTemps(tempNames);
    }

	public int computeMaximumCallArguments() {
		int l = stmt.computeMaximumCallArguments();
		int r = expr.computeMaximumCallArguments();
		return Math.max(l, r);
	}

	public int computeMaximumCallResults() {
		int l = stmt.computeMaximumCallResults();
		int r = expr.computeMaximumCallResults();
		return Math.max(l, r);
	}

    public int nodeCount() {
    	return 1 + stmt.nodeCount() + expr.nodeCount();
    }

    public Copyable copy() {
		return new IRESeq(stmt, expr);
    }
    
    public Copyable deepCopy() {
		return new IRESeq((IRStmt) stmt.deepCopy(), (IRExpr) expr.deepCopy());
    }

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
				stmt, expr
		});
	}
	
	public boolean equalsTree(Object object) {
		if (!(object instanceof IRESeq))
			return false;
		IRESeq eseq = (IRESeq) object;
		return eseq.stmt.equalsTree(stmt) && eseq.expr.equalsTree(expr);
	}
	
	public int treeHashCode() {
		return 17 + stmt.treeHashCode() + expr.treeHashCode() * 37;
	}
}
