package xic.parser;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.Map;

import polyglot.util.CodeWriter;
import polyglot.util.OptimalCodeWriter;
import xic.ast.Block;
import xic.ast.DeclBlock;
import xic.ast.FuncDecl;
import xic.ast.TypeNode;
import xic.ast.Typed;
import edu.cornell.cs.cs4120.util.TreePrinter;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

public class TypedASTPrinter implements TreePrinter {
	protected CodeWriter writer;

	public TypedASTPrinter(CodeWriter writer) {
		this.writer = writer;
	}

	public TypedASTPrinter(OutputStream o) {
		this.writer = new OptimalCodeWriter(o, 80);
	}

	public void print(VisualizableTreeNode node) {
		printHelper(node);
		try {
			writer.newline();
			writer.flush();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	private void printHelper(VisualizableTreeNode node) {
		writer.write("(");
		writer.write(node.label());
		if (node instanceof Typed)
			writer.write("<"+((Typed) node).getType()+">");
		for (VisualizableTreeNode child : node.children()) {
			writer.allowBreak(4);
			writer.begin(4);
			printHelper(child);
			writer.end();
		}
		if (node instanceof Block || node instanceof DeclBlock || node instanceof FuncDecl) {
			Map<String, TypeNode> symbolTable;
			if (node instanceof Block)
				symbolTable = ((Block) node).getSymbolTable();
			else if (node instanceof FuncDecl)
				symbolTable = ((FuncDecl) node).getSymbolTable();
			else
				symbolTable = ((DeclBlock) node).getSymbolTable();
			writer.allowBreak(4);
			if (symbolTable != null) {
				writer.begin(4);
				writer.write("(localSymbols={");
				if (symbolTable.size() > 0)
					writer.newline();
				for (Iterator<String> it = symbolTable.keySet().iterator(); it.hasNext(); ) {
					String key = it.next();
					TypeNode type = symbolTable.get(key);
					writer.write("'"+key+"' -> " + type.toString());
					if (it.hasNext()) {
						writer.write(", ");
						writer.newline();
					}
				}
				writer.end();
			}
			writer.write("})");
		}
		writer.write(")");
//		writer.allowBreak(0);
	}
}
